//
//  $Id: Prefix.pike,v 1.6 2002/10/21 20:25:02 jnt Exp $
//

//! This module describes an IP prefix, ie a network/netmask pair.

static object ip;
static object mask;
static int len;
static inherit "helpers";

//! Clone the IP.v4.Prefix module.
//!
//! @param prefix
//!    The IP prefix in the CIDR format (ie x.x.x.x/m)
//!
void create(string prefix) {
  string _ip, _mask;
  _ip = (prefix / "/")[0];
  _mask = (prefix / "/")[1];
  ip = IP.v4.Address(_ip);
  if (sizeof(_mask / ".") == 1) {
    len = (int)_mask;
    mask = IP.v4.Address(lengthtoint((int)_mask));
  }
  else {
    mask = IP.v4.Address(_mask);
    len = masktolength(mask);
  }
}

string _sprintf() {
  return sprintf("IP.v4.Prefix(\"%s/%s\")", (string)network(), (string)mask);
}

void|string cast(string type) {
  switch(type) {
  case "string":
    return (string)ip + "/" + (string)mask;
  }
}

//! Get the network address of this prefix.
IP.v4.Address network() {
  return IP.v4.Address(::network((int)ip, (int)mask));
}

//! Get the broadcast address of this prefix.
IP.v4.Address broadcast() {
  return IP.v4.Address(::broadcast((int)ip, (int)mask));
}

//! Get the netmask of this prefix.
IP.v4.Address netmask() {
  return mask;
}

//! Get the length of this prefix (ie, number of "on" bits in the mask).
int(0..32) length() {
  return len;
}

//! Test if an IP address is inside this prefix.
//! Returns 1 if true, 0 otherwise.
//!
//! @param _ip
//!    The IP address to test.
//!
int(0..1) contains(IP.v4.Address _ip) {
  return ((int)_ip >= (int)network()) && ((int)_ip <= (int)broadcast());
}
