//
//  $Id: Tuple.pike,v 1.2 2002/10/21 20:25:02 jnt Exp $
//

//! This module defines a connection between two machines, for example a TCP
//! socket, or a UDP datagram.

//! The local address of the datagram or stream.
static IP.v4.Address _local_addr;

//! The remote address of the datagram or stream.
static IP.v4.Address _remote_addr;

//! The local port of the datagram or streamm if applicable.
static int _local_port;

//! The remote port of the datagram or stream, if applicable.
static int _remote_port;

//! The IP protocol used for this datagram or stream.
static IP.v4.Protocol _protocol;

//! Clone the IP.v4.Tuple class.
//!
//! @param l_addr
//!   The local address.
//!
//! @param l_port
//!   The local port, if applicable.
//!
//! @param r_addr
//!   The remote address.
//!
//! @param r_port
//!   The remote port.
//!
//! @param proto
//!   The IP protocol.
//!
void create(Stdio.File|IP.v4.Address l_addr, void|int l_port, void|IP.v4.Address r_addr, void|int r_port, void|IP.v4.Protocol proto) {
  if (l_addr->stat)
    from_fd(l_addr);
  else {
    _local_addr = l_addr;
    _remote_addr = r_addr;
    _local_port = l_port;
    _remote_port = r_port;
    _protocol = proto;
  }
}

static void from_fd(Stdio.File fd) {
  string l_addr, r_addr;
  int l_port, r_port;
  sscanf(fd->query_address(), "%s %d", r_addr, r_port);
  sscanf(fd->query_address(1), "%s %d", l_addr, l_port);
  _local_addr = IP.v4.Address(l_addr);
  _local_port = l_port;
  _remote_addr = IP.v4.Address(r_addr);
  _remote_port = r_port;
  _protocol = IP.v4.Protocol("TCP");
}

string _sprintf() {
  return
    sprintf("IP.v4.Tuple(%O, %O, %O, %O, %O)",
      (string)_local_addr,
      (int)_local_port,
      (string)_remote_addr,
      (int)_remote_port,
      (string)_protocol
    );
}

IP.v4.Address local_addr(void|int|string|IP.v4.Address l_addr) {
  if (objectp(l_addr)) {
    if (l_addr->numeric && (l_addr->numeric() != _local_addr->numeric()))
      _local_addr = l_addr;
    return _local_addr;
  }
  else if (stringp(l_addr) || intp(l_addr)) {
    if (IP.v4.Address(l_addr)->numeric() != _local_addr->numeric())
      _local_addr = l_addr;
    return _local_addr;
  }
  else 
    return _local_addr;
}

IP.v4.Address remote_addr(void|int|string|IP.v4.Address r_addr) {
  if (objectp(r_addr)) {
    if (r_addr->numeric && (r_addr->numeric() != _remote_addr->numeric()))
      _remote_addr = r_addr;
    return _remote_addr;
  }
  else if (stringp(r_addr) || intp(r_addr)) {
    if (IP.v4.Address(r_addr)->numeric() != _remote_addr->numeric())
      _remote_addr = IP.v4.Address(r_addr);
    return _remote_addr;
  }
  else 
    return _remote_addr;
}

int local_port(int p) {
  if (p != _local_port)
    _local_port = p;
  return _local_port;
}

int remote_port(int p) {
  if (p != _remote_port)
    _remote_port = p;
  return _remote_port;
}

IP.v4.Protocol protocol(void|int|string|IP.v4.Protocol p) {
  if (objectp(p)) {
    if (p->name && (p->name() != _protocol->name()))
      _protocol = p;
    return _protocol;
  }
  else if (stringp(p) || intp(p)) {
    _protocol = IP.v4.Protocol(p);
    return _protocol;
  }
  else
    return _protocol;
}
