//
//  $Id: helpers.pike,v 1.3 2002/10/21 20:25:02 jnt Exp $
//

//! This module provides handy helpers to the IP.v4 modules.

//! Convert an IP address to a 32 bit integer.
//!
//! @param _ip
//!   The IP address to convert.
int iptoint(string _ip) {
  array ip = _ip / ".";
  return ((int)ip[0] << 24) + ((int)ip[1] << 16) + ((int)ip[2] << 8) + (int)ip[3];
}

//! Convert a 32 bit integer to an IP address.
//!
//! @param _ip
//!   The IP address to convert.
string inttoip(int _ip) {
  string ip = "";
  ip += (string)(_ip >> 24);
  ip += ".";
  ip += (string)((_ip >> 16) - ((_ip >> 24) * 256));
  ip += ".";
  ip += (string)((_ip >> 8) - ((_ip >> 16) * 256));
  ip += ".";
  ip += (string)((_ip) - ((_ip >> 8) * 256));
  return ip;
}

//! Find the network address for the given address and mask.
//!
//! @param ip
//!   An IP address.
//!
//! @param mask
//!   A netmask.
int network(int ip, int mask) {
  return ip & mask;
}

//! Find the broadcast address for the given address and mask.
//!
//! @param ip
//!   An IP address.
//!
//! @param mask
//!   A netmask.
int broadcast(int ip, int mask) {
  int net = ip & mask;
  int _msk = mask ^ 4294967295;
  return net | _msk;
}

//! Convert a prefix length into an integer mask.
//!
//! @param length
//!   A prefix length.
int lengthtoint(int length) {
  if (length == 32)
    return 4294967295;
  else if (length < 32)
    return (int)((int)pow(2,32) - (int)pow(2,(32 -length)));
}

//! Convert a mask to a prefix length.
//!
//! @param mask
//!   The netmask.
int masktolength(int|object mask) {
  for(int i = 0; i <= 32; i++) {
    if (lengthtoint(i) == (int)mask)
      return i;
  }
}
